varying vec3 v_v3ViewDir;
varying vec3 v_v3LightDir;
varying vec3 v_v3Normal;
uniform float u_fTime;
uniform sampler2D u_s2Reflection;
uniform sampler2D u_s2Refraction;
uniform sampler2D u_s2Bump;
uniform vec2 u_v2Screen;


void main(void)
{
	// create screenspace-coordinates
	vec2 v2TexCoord   = gl_FragCoord.xy/vec2(u_v2Screen.x, u_v2Screen.y);

	// calculate diffuse-strength and normal-offsets
	vec3 v3MathLightDir = normalize(v_v3LightDir);
	vec3 v3TexNormal1   = normalize((texture2D(u_s2Bump, gl_TexCoord[0].st) * 2.0f) - 1.0f).xyz;
	vec3 v3TexNormal2   = normalize((texture2D(u_s2Bump, gl_TexCoord[1].st) * 2.0f) - 1.0f).xyz;
	vec3 v3TexNormal3   = normalize((texture2D(u_s2Bump, gl_TexCoord[2].st) * 2.0f) - 1.0f).xyz;
	vec3 v3MathNormal   = normalize(v_v3Normal);
	vec3 v3BumpNormal   = normalize(v3TexNormal1*5.0f + vec3(v3MathNormal.x, 0.0f, v3MathNormal.z)*1.3f + v3TexNormal3*0.3f);
	float fBumpFactor   = dot(v3MathLightDir, v3BumpNormal); 

	// calculate specular-strength
	vec3 v3MathViewDir = normalize(v_v3ViewDir);
	vec3 v3ReflNormal  = normalize(((2.0f * v3BumpNormal) * fBumpFactor) - v3MathLightDir); 
	float fReflFactor  = max(0.0f, dot(v3MathViewDir, v3ReflNormal));
	
	// read reflection and refraction textures (with xy, not xz)
	vec4 v4ReflectionTexel = texture2D(u_s2Reflection, clamp(v2TexCoord + (v3TexNormal1.xy + v3TexNormal2.xy) * gl_FragCoord.z * 0.012f, 0.001f, 0.999f) * vec2(1.0f,-1.0f));
	vec4 v4RefractionTexel = texture2D(u_s2Refraction, v2TexCoord);

	// amplify geometry-normal
	v3BumpNormal = normalize(v3BumpNormal + v3MathNormal*2.0f);

	// calculate color-values
	vec4 v4Color		 = mix(v4RefractionTexel, v4ReflectionTexel, abs(dot(v3BumpNormal, vec3(0.0f,1.0f,0.0f)))+0.1f);
	vec4 v4FinalDiffuse  = 1.0f * v4Color * (abs(dot(v3MathLightDir, v3BumpNormal))*0.5f+0.7f);
	vec4 v4FinalSpecular = 0.5f * vec4(1.0f, 1.0f, 1.0f, 1.0f) * pow(fReflFactor, 25.0f);
	vec4 v4FinalAmbient  = 0.3f * vec4(0.122f, 0.261f, 0.394f, 1.0f); 

	// set final color-value
	gl_FragColor = v4FinalDiffuse + v4FinalSpecular + v4FinalAmbient;
	gl_FragColor.a = 1.0f;
}